@@ -12,6 +12,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import absolute_import
+
 from kafka_utils.util.error import KafkaToolError
 
 
@@ -25,6 +27,18 @@ class InvalidPartitionError(KafkaToolError):
     pass
 
 
+class InvalidReplicationFactorError(KafkaToolError):
+    """Raised when an operation would result in an replication factor that is
+    too small or too large for the cluster.
+    """
+    pass
+
+
+class InvalidPartitionMeasurementError(KafkaToolError):
+    """Raised when a partition is assigned a negative weight or size."""
+    pass
+
+
 class EmptyReplicationGroupError(KafkaToolError):
     """Raised when there are no brokers in a replication group."""
     pass
