@@ -12,20 +12,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import absolute_import
+
 from collections import defaultdict
 from collections import namedtuple
 
-from kafka.common import BrokerResponseError
-from kafka.common import check_error
-from kafka.common import OffsetCommitRequest
-from kafka.common import OffsetFetchRequest
-from kafka.common import OffsetFetchResponse
-from kafka.common import OffsetRequest
-from kafka.common import OffsetResponse
+import six
 from kafka.common import UnknownTopicOrPartitionError
-from kafka.util import kafka_bytestring
+from kafka.errors import BrokerResponseError
+from kafka.errors import check_error
+from kafka.structs import OffsetCommitRequestPayload
+from kafka.structs import OffsetFetchRequestPayload
+from kafka.structs import OffsetFetchResponsePayload
+from kafka.structs import OffsetRequestPayload
+from kafka.structs import OffsetResponsePayload
 
-from kafka_utils.util.error import InvalidOffsetStorageError
 from kafka_utils.util.error import OffsetCommitError
 from kafka_utils.util.error import UnknownPartitions
 from kafka_utils.util.error import UnknownTopic
@@ -57,7 +58,7 @@ def pluck_topic_offset_or_zero_on_unknown(resp):
     # The API spec says server wont set an error, but 0.8.1.1 does. The actual
     # check is if the offset is -1.
     if resp.offset == -1:
-        return OffsetFetchResponse(
+        return OffsetFetchResponsePayload(
             resp.topic,
             resp.partition,
             0,
@@ -72,7 +73,7 @@ def _check_fetch_response_error(resp):
         check_error(resp)
     except BrokerResponseError:
         # In case of error we set the offset to (-1,)
-        return OffsetResponse(
+        return OffsetResponsePayload(
             resp.topic,
             resp.partition,
             resp.error,
@@ -91,6 +92,7 @@ def _check_commit_response_error(resp):
             e.message
         )
         return exception
+    return resp
 
 
 def _validate_topics_list_or_dict(topics):
@@ -107,7 +109,7 @@ def _validate_topics_list_or_dict(topics):
 def _verify_topics_and_partitions(kafka_client, topics, raise_on_error):
     topics = _validate_topics_list_or_dict(topics)
     valid_topics = {}
-    for topic, partitions in topics.iteritems():
+    for topic, partitions in six.iteritems(topics):
         # Check topic exists
         if not kafka_client.has_metadata_for_topic(topic):
             if raise_on_error:
@@ -148,13 +150,13 @@ def _verify_commit_offsets_requests(kafka_client, new_offsets, raise_on_error):
     if not isinstance(new_offsets, dict):
         raise TypeError(type_error_str)
 
-    for topic, partitions in new_offsets.iteritems():
+    for topic, partitions in six.iteritems(new_offsets):
         if not isinstance(partitions, dict):
             raise TypeError(type_error_str)
 
     topics = dict(
-        (topic, partitions.keys())
-        for topic, partitions in new_offsets.iteritems()
+        (topic, list(partitions.keys()))
+        for topic, partitions in six.iteritems(new_offsets)
     )
 
     valid_topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
@@ -164,7 +166,7 @@ def _verify_commit_offsets_requests(kafka_client, new_offsets, raise_on_error):
             (partition, new_offsets[topic][partition])
             for partition in partitions
         ))
-        for topic, partitions in valid_topics.iteritems()
+        for topic, partitions in six.iteritems(valid_topics)
         if partitions
     )
 
@@ -174,7 +176,6 @@ def get_current_consumer_offsets(
     group,
     topics,
     raise_on_error=True,
-    offset_storage='zookeeper',
 ):
     """ Get current consumer offsets.
 
@@ -190,7 +191,6 @@ def get_current_consumer_offsets(
     :param topics: topic list or dict {<topic>: [partitions]}
     :param raise_on_error: if False the method ignores missing topics and
       missing partitions. It still may fail on the request send.
-    :param offset_storage: String, one of {zookeeper, kafka}.
     :returns: a dict topic: partition: offset
     :raises:
       :py:class:`kafka_utils.util.error.UnknownTopic`: upon missing
@@ -199,33 +199,25 @@ def get_current_consumer_offsets(
       :py:class:`kafka_utils.util.error.UnknownPartition`: upon missing
       partitions and raise_on_error=True
 
-      :py:class:`kafka_utils.util.error.InvalidOffsetStorageError: upon unknown
-      offset_storage choice.
-
       FailedPayloadsError: upon send request error.
     """
 
     topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
 
     group_offset_reqs = [
-        OffsetFetchRequest(kafka_bytestring(topic), partition)
-        for topic, partitions in topics.iteritems()
+        OffsetFetchRequestPayload(topic, partition)
+        for topic, partitions in six.iteritems(topics)
         for partition in partitions
     ]
 
     group_offsets = {}
 
-    if offset_storage == 'zookeeper':
-        send_api = kafka_client.send_offset_fetch_request
-    elif offset_storage == 'kafka':
-        send_api = kafka_client.send_offset_fetch_request_kafka
-    else:
-        raise InvalidOffsetStorageError(offset_storage)
+    send_api = kafka_client.send_offset_fetch_request_kafka
 
     if group_offset_reqs:
         # fail_on_error = False does not prevent network errors
         group_resps = send_api(
-            group=kafka_bytestring(group),
+            group=group,
             payloads=group_offset_reqs,
             fail_on_error=False,
             callback=pluck_topic_offset_or_zero_on_unknown,
@@ -271,19 +263,19 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
     highmark_offset_reqs = []
     lowmark_offset_reqs = []
 
-    for topic, partitions in topics.iteritems():
+    for topic, partitions in six.iteritems(topics):
         # Batch watermark requests
         for partition in partitions:
             # Request the the latest offset
             highmark_offset_reqs.append(
-                OffsetRequest(
-                    kafka_bytestring(topic), partition, -1, max_offsets=1
+                OffsetRequestPayload(
+                    topic, partition, -1, max_offsets=1
                 )
             )
             # Request the earliest offset
             lowmark_offset_reqs.append(
-                OffsetRequest(
-                    kafka_bytestring(topic), partition, -2, max_offsets=1
+                OffsetRequestPayload(
+                    topic, partition, -2, max_offsets=1
                 )
             )
 
@@ -314,8 +306,8 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
         aggregated_offsets[resp.topic][resp.partition]['lowmark'] = \
             resp.offsets[0]
 
-    for topic, partition_watermarks in aggregated_offsets.iteritems():
-        for partition, watermarks in partition_watermarks.iteritems():
+    for topic, partition_watermarks in six.iteritems(aggregated_offsets):
+        for partition, watermarks in six.iteritems(partition_watermarks):
             watermark_offsets.setdefault(
                 topic,
                 {},
@@ -334,7 +326,6 @@ def _commit_offsets_to_watermark(
     topics,
     watermark,
     raise_on_error,
-    offset_storage,
 ):
     topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
 
@@ -342,20 +333,22 @@ def _commit_offsets_to_watermark(
 
     if watermark == HIGH_WATERMARK:
         group_offset_reqs = [
-            OffsetCommitRequest(
-                kafka_bytestring(topic), partition,
-                watermark_offsets[topic][partition].highmark, None
+            OffsetCommitRequestPayload(
+                topic, partition,
+                watermark_offsets[topic][partition].highmark,
+                metadata=''
             )
-            for topic, partitions in topics.iteritems()
+            for topic, partitions in six.iteritems(topics)
             for partition in partitions
         ]
     elif watermark == LOW_WATERMARK:
         group_offset_reqs = [
-            OffsetCommitRequest(
-                kafka_bytestring(topic), partition,
-                watermark_offsets[topic][partition].lowmark, None
+            OffsetCommitRequestPayload(
+                topic, partition,
+                watermark_offsets[topic][partition].lowmark,
+                metadata=''
             )
-            for topic, partitions in topics.iteritems()
+            for topic, partitions in six.iteritems(topics)
             for partition in partitions
         ]
     else:
@@ -363,23 +356,18 @@ def _commit_offsets_to_watermark(
             "Unknown watermark: {watermark}".format(watermark=watermark)
         )
 
-    if offset_storage == 'zookeeper' or not offset_storage:
-        send_api = kafka_client.send_offset_commit_request
-    elif offset_storage == 'kafka':
-        send_api = kafka_client.send_offset_commit_request_kafka
-    else:
-        raise InvalidOffsetStorageError(offset_storage)
+    send_api = kafka_client.send_offset_commit_request_kafka
 
     status = []
     if group_offset_reqs:
         status = send_api(
-            kafka_bytestring(group),
+            group,
             group_offset_reqs,
             raise_on_error,
             callback=_check_commit_response_error
         )
 
-    return filter(None, status)
+    return [_f for _f in status if _f]
 
 
 def advance_consumer_offsets(
@@ -387,7 +375,6 @@ def advance_consumer_offsets(
     group,
     topics,
     raise_on_error=True,
-    offset_storage='zookeeper',
 ):
     """Advance consumer offsets to the latest message in the topic
     partition (the high watermark).
@@ -404,7 +391,6 @@ def advance_consumer_offsets(
     :param topics: topic list or dict {<topic>: [partitions]}
     :param raise_on_error: if False the method does not raise exceptions
       on missing topics/partitions. It may still fail on the request send.
-    :param offset_storage: String, one of {zookeeper, kafka}.
     :returns: a list of errors for each partition offset update that failed.
     :rtype: list [OffsetCommitError]
     :raises:
@@ -421,7 +407,6 @@ def advance_consumer_offsets(
     return _commit_offsets_to_watermark(
         kafka_client, group, topics,
         HIGH_WATERMARK, raise_on_error,
-        offset_storage,
     )
 
 
@@ -430,7 +415,6 @@ def rewind_consumer_offsets(
     group,
     topics,
     raise_on_error=True,
-    offset_storage='zookeeper',
 ):
     """Rewind consumer offsets to the earliest message in the topic
     partition (the low watermark).
@@ -447,7 +431,6 @@ def rewind_consumer_offsets(
     :param topics: topic list or dict {<topic>: [partitions]}
     :param raise_on_error: if False the method does not raise exceptions
       on missing topics/partitions. It may still fail on the request send.
-    :param offset_storage: String, one of {zookeeper, kafka}.
     :returns: a list of errors for each partition offset update that failed.
     :rtype: list [OffsetCommitError]
     :raises:
@@ -464,7 +447,6 @@ def rewind_consumer_offsets(
     return _commit_offsets_to_watermark(
         kafka_client, group, topics,
         LOW_WATERMARK, raise_on_error,
-        offset_storage,
     )
 
 
@@ -473,7 +455,6 @@ def set_consumer_offsets(
     group,
     new_offsets,
     raise_on_error=True,
-    offset_storage='zookeeper',
 ):
     """Set consumer offsets to the specified offsets.
 
@@ -489,7 +470,6 @@ def set_consumer_offsets(
     :param topics: dict {<topic>: {<partition>: <offset>}}
     :param raise_on_error: if False the method does not raise exceptions
       on errors encountered. It may still fail on the request send.
-    :param offset_storage: String, one of {zookeeper, kafka}.
     :returns: a list of errors for each partition offset update that failed.
     :rtype: list [OffsetCommitError]
     :raises:
@@ -502,9 +482,6 @@ def set_consumer_offsets(
       :py:class:`exceptions.TypeError`: upon badly formatted input
       new_offsets
 
-      :py:class:`kafka_utils.util.error.InvalidOffsetStorageError: upon unknown
-      offset_storage choice.
-
       FailedPayloadsError: upon send request error.
     """
     valid_new_offsets = _verify_commit_offsets_requests(
@@ -514,39 +491,35 @@ def set_consumer_offsets(
     )
 
     group_offset_reqs = [
-        OffsetCommitRequest(
-            kafka_bytestring(topic),
+        OffsetCommitRequestPayload(
+            topic,
             partition,
             offset,
-            None
+            metadata='',
         )
-        for topic, new_partition_offsets in valid_new_offsets.iteritems()
-        for partition, offset in new_partition_offsets.iteritems()
+        for topic, new_partition_offsets in six.iteritems(valid_new_offsets)
+        for partition, offset in six.iteritems(new_partition_offsets)
     ]
 
-    if offset_storage == 'zookeeper' or not offset_storage:
-        send_api = kafka_client.send_offset_commit_request
-    elif offset_storage == 'kafka':
-        send_api = kafka_client.send_offset_commit_request_kafka
-    else:
-        raise InvalidOffsetStorageError(offset_storage)
+    send_api = kafka_client.send_offset_commit_request_kafka
 
     status = []
     if group_offset_reqs:
         status = send_api(
-            kafka_bytestring(group),
+            group,
             group_offset_reqs,
             raise_on_error,
             callback=_check_commit_response_error
         )
 
-    return filter(None, status)
+    return [_f for _f in status
+            if _f and _f.error != 0]
 
 
 def _nullify_partition_offsets(partition_offsets):
     result = {}
     for partition in partition_offsets:
-        result[partition] = -1
+        result[partition] = 0
     return result
 
 
@@ -558,6 +531,6 @@ def nullify_offsets(offsets):
     :returns: a dict topic: partition: offset
     """
     result = {}
-    for topic, partition_offsets in offsets.iteritems():
+    for topic, partition_offsets in six.iteritems(offsets):
         result[topic] = _nullify_partition_offsets(partition_offsets)
     return result
