@@ -12,13 +12,18 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import json
+from __future__ import absolute_import
+
 import logging
 
+import six
 from kazoo.client import KazooClient
 from kazoo.exceptions import NodeExistsError
 from kazoo.exceptions import NoNodeError
+from kazoo.retry import KazooRetry
 
+from kafka_utils.util.serialization import dump_json
+from kafka_utils.util.serialization import load_json
 from kafka_utils.util.validation import validate_plan
 
 
@@ -35,9 +40,13 @@ class ZK:
         self.cluster_config = cluster_config
 
     def __enter__(self):
+        kazooRetry = KazooRetry(
+            max_tries=5,
+        )
         self.zk = KazooClient(
             hosts=self.cluster_config.zookeeper,
             read_only=True,
+            connection_retry=kazooRetry,
         )
         _log.debug(
             "ZK: Creating new zookeeper connection: {zookeeper}"
@@ -73,7 +82,7 @@ class ZK:
     def get_json(self, path, watch=None):
         """Reads the data of the specified node and converts it to json."""
         data, _ = self.get(path, watch)
-        return json.loads(data) if data else None
+        return load_json(data) if data else None
 
     def get_broker_metadata(self, broker_id):
         try:
@@ -85,15 +94,20 @@ class ZK:
                 "broker '{b_id}' not found.".format(b_id=broker_id),
             )
             raise
-        return json.loads(broker_json)
+        return load_json(broker_json)
 
     def get_brokers(self, names_only=False):
         """Get information on all the available brokers.
 
         :rtype : dict of brokers
         """
-        broker_ids = self.get_children("/brokers/ids")
-
+        try:
+            broker_ids = self.get_children("/brokers/ids")
+        except NoNodeError:
+            _log.info(
+                "cluster is empty."
+            )
+            return {}
         # Return broker-ids only
         if names_only:
             return {int(b_id): None for b_id in broker_ids}
@@ -105,33 +119,72 @@ class ZK:
         :rtype : dict of configuration
         """
         try:
-            config_data = json.loads(
+            config_data = load_json(
                 self.get(
                     "/config/topics/{topic}".format(topic=topic)
                 )[0]
             )
         except NoNodeError as e:
-            _log.error(
-                "topic {topic} not found.".format(topic=topic)
-            )
-            raise e
+
+            # Kafka version before 0.8.1 does not have "/config/topics/<topic_name>" path in ZK and
+            # if the topic exists, return default dict instead of raising an Exception.
+            # Ref: https://cwiki.apache.org/confluence/display/KAFKA/Kafka+data+structures+in+Zookeeper.
+
+            topics = self.get_topics(topic_name=topic, fetch_partition_state=False)
+            if len(topics) > 0:
+                _log.info("Configuration not available for topic {topic}.".format(topic=topic))
+                config_data = {"config": {}}
+            else:
+                _log.error(
+                    "topic {topic} not found.".format(topic=topic)
+                )
+                raise e
         return config_data
 
-    def set_topic_config(self, topic, value):
+    def set_topic_config(self, topic, value, kafka_version=(0, 10, )):
         """Set configuration information for specified topic.
 
-        :rtype : dict of new configuration"""
+        :topic : topic whose configuration needs to be changed
+        :value :  config value with which the topic needs to be
+            updated with. This would be of the form key=value.
+            Example 'cleanup.policy=compact'
+        :kafka_version :tuple kafka version the brokers are running on.
+            Defaults to (0, 10, x). Kafka version 9 and kafka 10
+            support this feature.
+        """
+        config_data = dump_json(value)
+
         try:
-            config_data = json.dumps(value)
             # Change value
             return_value = self.set(
                 "/config/topics/{topic}".format(topic=topic),
                 config_data
             )
             # Create change
+            version = kafka_version[1]
+
+            # this feature is supported in kafka 9 and kafka 10
+            assert version in (9, 10), "Feature supported with kafka 9 and kafka 10"
+
+            if version == 9:
+                # https://github.com/apache/kafka/blob/0.9.0.1/
+                #     core/src/main/scala/kafka/admin/AdminUtils.scala#L334
+                change_node = dump_json({
+                    "version": 1,
+                    "entity_type": "topics",
+                    "entity_name": topic
+                })
+            else:  # kafka 10
+                # https://github.com/apache/kafka/blob/0.10.2.1/
+                #     core/src/main/scala/kafka/admin/AdminUtils.scala#L574
+                change_node = dump_json({
+                    "version": 2,
+                    "entity_path": "topics/" + topic,
+                })
+
             self.create(
                 '/config/changes/config_change_',
-                topic,
+                change_node,
                 sequence=True
             )
         except NoNodeError as e:
@@ -162,6 +215,7 @@ class ZK:
         Topic-data format with fetch_partition_state as True:-
         topic_data = {
             'version': 1,
+            'ctime': <timestamp>,
             'partitions': {
                 <p_id>:{
                     replicas: [<broker_id>, <broker_id>, ...],
@@ -170,35 +224,51 @@ class ZK:
                     leader_epoch: <val>,
                     version: 1,
                     leader: <broker-id>,
+                    ctime: <timestamp>,
+                }
             }
         }
         Note: By default we also fetch partition-state which results in
         accessing the zookeeper twice. If just partition-replica information is
         required fetch_partition_state should be set to False.
         """
-        topic_ids = [topic_name] if topic_name else self.get_children(
-            "/brokers/topics",
-        )
+        try:
+            topic_ids = [topic_name] if topic_name else self.get_children(
+                "/brokers/topics",
+            )
+        except NoNodeError:
+            _log.error(
+                "Cluster is empty."
+            )
+            return {}
+
         if names_only:
             return topic_ids
         topics_data = {}
         for topic_id in topic_ids:
             try:
-                topic_data = json.loads(
-                    self.get("/brokers/topics/{id}".format(id=topic_id))[0],
-                )
+                topic_info = self.get("/brokers/topics/{id}".format(id=topic_id))
+                topic_data = load_json(topic_info[0])
+                topic_ctime = topic_info[1].ctime / 1000.0
+                topic_data['ctime'] = topic_ctime
             except NoNodeError:
-                _log.error(
+                _log.info(
                     "topic '{topic}' not found.".format(topic=topic_id),
                 )
                 return {}
             # Prepare data for each partition
             partitions_data = {}
-            for p_id, replicas in topic_data['partitions'].iteritems():
+            for p_id, replicas in six.iteritems(topic_data['partitions']):
                 partitions_data[p_id] = {}
                 if fetch_partition_state:
                     # Fetch partition-state from zookeeper
-                    partitions_data[p_id] = self._fetch_partition_state(topic_id, p_id)
+                    partition_state = self._fetch_partition_state(topic_id, p_id)
+                    partitions_data[p_id] = load_json(partition_state[0])
+                    partitions_data[p_id]['ctime'] = partition_state[1].ctime / 1000.0
+                else:
+                    # Fetch partition-info from zookeeper
+                    partition_info = self._fetch_partition_info(topic_id, p_id)
+                    partitions_data[p_id]['ctime'] = partition_info.ctime / 1000.0
                 partitions_data[p_id]['replicas'] = replicas
             topic_data['partitions'] = partitions_data
             topics_data[topic_id] = topic_data
@@ -300,7 +370,7 @@ class ZK:
                 try:
                     # Get current offset
                     offset_json, _ = self.get(path)
-                    group_offsets[topic][partition] = json.loads(offset_json)
+                    group_offsets[topic][partition] = load_json(offset_json)
                 except NoNodeError:
                     _log.error("Path {path} not found".format(path=path))
                     raise
@@ -310,10 +380,21 @@ class ZK:
         """Fetch partition-state for given topic-partition."""
         state_path = "/brokers/topics/{topic_id}/partitions/{p_id}/state"
         try:
-            partition_json, _ = self.get(
+            partition_state = self.get(
                 state_path.format(topic_id=topic_id, p_id=partition_id),
             )
-            return json.loads(partition_json)
+            return partition_state
+        except NoNodeError:
+            return {}  # The partition has no data
+
+    def _fetch_partition_info(self, topic_id, partition_id):
+        """Fetch partition info for given topic-partition."""
+        info_path = "/brokers/topics/{topic_id}/partitions/{p_id}"
+        try:
+            _, partition_info = self.get(
+                info_path.format(topic_id=topic_id, p_id=partition_id),
+            )
+            return partition_info
         except NoNodeError:
             return {}  # The partition has no data
 
@@ -386,26 +467,6 @@ class ZK:
         _log.debug("ZK: Deleting node " + path)
         return self.zk.delete(path, recursive=recursive)
 
-    def delete_topic_partitions(self, groupid, topic, partitions):
-        """Delete the specified partitions within the topic that the consumer
-        is subscribed to.
-
-        :param: groupid: The consumer group ID for the consumer.
-        :param: topic: Kafka topic.
-        :param: partitions: List of partitions within the topic to be deleted.
-        :raises:
-          NoNodeError: if the consumer is not subscribed to the topic
-
-          ZookeeperError: if there is an error with Zookeeper
-        """
-        for partition in partitions:
-            path = "/consumers/{groupid}/offsets/{topic}/{partition}".format(
-                groupid=groupid,
-                topic=topic,
-                partition=partition
-            )
-            self.delete(path)
-
     def delete_topic(self, groupid, topic):
         path = "/consumers/{groupid}/offsets/{topic}".format(
             groupid=groupid,
@@ -419,14 +480,14 @@ class ZK:
         )
         self.delete(path, True)
 
-    def execute_plan(self, plan):
+    def execute_plan(self, plan, allow_rf_change=False):
         """Submit reassignment plan for execution."""
         reassignment_path = '{admin}/{reassignment_node}'\
             .format(admin=ADMIN_PATH, reassignment_node=REASSIGNMENT_NODE)
-        plan_json = json.dumps(plan)
+        plan_json = dump_json(plan)
         base_plan = self.get_cluster_plan()
-        if not validate_plan(plan, base_plan):
-            _log.error('Given plan is invalid. ABORTING reassignment...')
+        if not validate_plan(plan, base_plan, allow_rf_change=allow_rf_change):
+            _log.error('Given plan is invalid. Aborting new reassignment plan ... {plan}'.format(plan=plan))
             return False
         # Send proposed-plan to zookeeper
         try:
@@ -439,7 +500,8 @@ class ZK:
             return True
         except NodeExistsError:
             _log.warning('Previous plan in progress. Exiting..')
-            in_progress_plan = json.loads(self.get(reassignment_path)[0])
+            _log.warning('Aborting new reassignment plan... {plan}'.format(plan=plan))
+            in_progress_plan = load_json(self.get(reassignment_path)[0])
             in_progress_partitions = [
                 '{topic}-{p_id}'.format(
                     topic=p_data['topic'],
@@ -452,7 +514,7 @@ class ZK:
                 .format(count=len(in_progress_partitions)),
             )
             _log.warning(
-                '{partitions}. ABORTING reassignment...'.format(
+                '{partitions}. In Progress reassignment plan...'.format(
                     partitions=', '.join(in_progress_partitions),
                 ),
             )
@@ -476,8 +538,8 @@ class ZK:
                 'partition': int(p_id),
                 'replicas': partitions_data['replicas']
             }
-            for topic_id, topic_info in cluster_layout.iteritems()
-            for p_id, partitions_data in topic_info['partitions'].iteritems()
+            for topic_id, topic_info in six.iteritems(cluster_layout)
+            for p_id, partitions_data in six.iteritems(topic_info['partitions'])
         ]
         return {
             'version': 1,
@@ -490,6 +552,6 @@ class ZK:
             .format(admin=ADMIN_PATH, reassignment_node=REASSIGNMENT_NODE)
         try:
             result = self.get(reassignment_path)
-            return json.loads(result[0])
+            return load_json(result[0])
         except NoNodeError:
             return {}
